<?php
/**
 * This file is part of PHP_Depend.
 *
 * PHP Version 5
 *
 * Copyright (c) 2008-2011, Manuel Pichler <mapi@pdepend.org>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   * Neither the name of Manuel Pichler nor the names of his
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  QualityAssurance
 * @package   PHP_Depend
 * @author    Manuel Pichler <mapi@pdepend.org>
 * @copyright 2008-2011 Manuel Pichler. All rights reserved.
 * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version   SVN: $Id$
 * @link      http://pdepend.org/
 */

require_once dirname(__FILE__) . '/AbstractTest.php';
require_once dirname(__FILE__) . '/AutoloadTest.php';
require_once dirname(__FILE__) . '/DependTest.php';
require_once dirname(__FILE__) . '/ParserTest.php';
require_once dirname(__FILE__) . '/ParserRegressionTest.php';
require_once dirname(__FILE__) . '/TokenTest.php';
require_once dirname(__FILE__) . '/Bugs/AllTests.php';
require_once dirname(__FILE__) . '/Builder/AllTests.php';
require_once dirname(__FILE__) . '/Code/AllTests.php';
require_once dirname(__FILE__) . '/Input/AllTests.php';
require_once dirname(__FILE__) . '/Integration/AllTests.php';
require_once dirname(__FILE__) . '/Issues/AllTests.php';
require_once dirname(__FILE__) . '/Log/AllTests.php';
require_once dirname(__FILE__) . '/Metrics/AllTests.php';
require_once dirname(__FILE__) . '/Parser/AllTests.php';
require_once dirname(__FILE__) . '/TextUI/AllTests.php';
require_once dirname(__FILE__) . '/Tokenizer/AllTests.php';
require_once dirname(__FILE__) . '/Util/AllTests.php';
require_once dirname(__FILE__) . '/Visitor/AllTests.php';

/**
 * Main test suite for the PHP_Depend package.
 *
 * @category  QualityAssurance
 * @package   PHP_Depend
 * @author    Manuel Pichler <mapi@pdepend.org>
 * @copyright 2008-2011 Manuel Pichler. All rights reserved.
 * @license   http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version   Release: @package_version@
 * @link      http://pdepend.org/
 */
class PHP_Depend_AllTests
{
    /**
     * Creates the phpunit test suite for this package.
     *
     * @return PHPUnit_Framework_TestSuite
     */
    public static function suite()
    {
        $suite = new PHPUnit_Framework_TestSuite('PHP_Depend - AllTests');
        $suite->addTestSuite('PHP_Depend_TokenTest');
        $suite->addTestSuite('PHP_Depend_AutoloadTest');

        $suite->addTest(PHP_Depend_Bugs_AllTests::suite());
        $suite->addTest(PHP_Depend_Builder_AllTests::suite());
        $suite->addTest(PHP_Depend_Code_AllTests::suite());
        $suite->addTest(PHP_Depend_Issues_AllTests::suite());
        $suite->addTest(PHP_Depend_Log_AllTests::suite());
        $suite->addTest(PHP_Depend_Input_AllTests::suite());
        $suite->addTest(PHP_Depend_Integration_AllTests::suite());
        $suite->addTest(PHP_Depend_Metrics_AllTests::suite());
        $suite->addTest(PHP_Depend_Parser_AllTests::suite());
        $suite->addTest(PHP_Depend_TextUI_AllTests::suite());
        $suite->addTest(PHP_Depend_Tokenizer_AllTests::suite());
        $suite->addTest(PHP_Depend_Util_AllTests::suite());
        $suite->addTest(PHP_Depend_Visitor_AllTests::suite());

        $suite->addTestSuite('PHP_Depend_ParserTest');
        $suite->addTestSuite('PHP_Depend_ParserRegressionTest');
        $suite->addTestSuite('PHP_Depend_DependTest');

        return $suite;
    }
}