<?php
// vim: set ts=4 sw=4 sts=4 et:

/**
 * LiteCommerce
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to licensing@litecommerce.com so we can send you a copy immediately.
 *
 * @category   LiteCommerce
 * @package    Tests
 * @subpackage Web
 * @author     Creative Development LLC <info@cdev.ru>
 * @copyright  Copyright (c) 2010 Creative Development LLC <info@cdev.ru>. All rights reserved
 * @license    http://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 * @link       http://www.litecommerce.com/
 * @see        ____file_see____
 * @since      1.0.0
 *
 * @block_all
 */

/**
 * Substutional skin
 *
 * @package XLite
 * @see     ____class_see____
 * @since   1.0.0
 */
class XLite_Web_Customer_SubstutionalSkin extends XLite_Web_Customer_ACustomer
{
    /**
     * PHPUnit default function.
     * Redefine this method only if you really need to do so.
     * In any other cases redefine the getRequest() one
     *
     * @return void
     * @access protected
     * @see    ____func_see____
     * @since  1.0.0
     */
    protected function setUp()
    {
        parent::setUp();

        \Includes\Utils\FileManager::copyRecursive(
            dirname(__FILE__) . LC_DS . 'SubstutionalSkin' . LC_DS . 'scripts' . LC_DS . 'CDev',
            LC_DIR_MODULES . 'CDev'
        );

        \Includes\Utils\FileManager::copyRecursive(
            dirname(__FILE__) . LC_DS . 'SubstutionalSkin' . LC_DS . 'scripts' . LC_DS . 'CDev',
            LC_DIR_CACHE_CLASSES . 'XLite' . LC_DS . 'Module' . LC_DS . 'CDev'
        );

        \Includes\Utils\FileManager::copyRecursive(
            dirname(__FILE__) . LC_DS . 'SubstutionalSkin' . LC_DS . 'skins',
            LC_DIR_SKINS
        );

        $skin = \XLite\Core\Database::getRepo('XLite\Model\Module')->findOneBy(
            array(
                'name'   => 'TestSkin',
                'author' => 'CDev',
            )
        );

        if (!$skin) {

            $skin = new \XLite\Model\Module;

            $skin->setAuthor('CDev');
            $skin->setName('TestSkin');
            $skin->setMajorVersion('1.0');
            $skin->setMinorVersion('0');
            $skin->setModuleName('TestSkin');
            $skin->setAuthorName('CDev');


            \XLite\Core\Database::getEM()->persist($skin);
        }

        $skin->setInstalled(true);
        $skin->setEnabled(true);

        \XLite\Core\Database::getEM()->flush();

        \XLite\Core\Database::getCacheDriver()->deleteAll();
        \Includes\Utils\FileManager::unlinkRecursive(
            LC_DIR_COMPILE . 'skins'
        );

    }

    /**
     * PHPUnit default function.
     * It's not recommended to redefine this method
     *
     * @return void
     * @access protected
     * @see    ____func_see____
     * @since  1.0.0
     */
    protected function tearDown()
    {
        \Includes\Utils\FileManager::unlinkRecursive(
            LC_DIR_MODULES . 'CDev' . LC_DS . 'TestSkin'
        );
        \Includes\Utils\FileManager::unlinkRecursive(
            LC_DIR_CACHE_CLASSES . 'XLite' . LC_DS . 'Module' . LC_DS . 'CDev' . LC_DS . 'TestSkin'
        );
        \Includes\Utils\FileManager::unlinkRecursive(
            LC_DIR_SKINS . 'test'
        );

        $skin = \XLite\Core\Database::getRepo('XLite\Model\Module')->findOneBy(
            array(
                'name'   => 'TestSkin',
                'author' => 'CDev',
            )
        );

        if ($skin) {
            \XLite\Core\Database::getEM()->remove($skin);
            \XLite\Core\Database::getEM()->flush();
        }

        \XLite\Core\Database::getCacheDriver()->deleteAll();
        \Includes\Utils\FileManager::unlinkRecursive(
            LC_DIR_COMPILE . 'skins'
        );

        parent::tearDown();
    }

    public function testSubstitute()
    {
        $this->openAndWait('');

        $this->assertElementPresent(
            "//h1[@class='substitutional-test-skin' and text()='WELCOME PAGE']",
            'test substutional template'
        );

        $length = $this->getJSExpression('jQuery(\'img[src*="default/en/images/spacer.gif"]\').length');
        $this->assertTrue(0 == $length, 'check spacer.gif substitute');
    }

    public function testSubstituteMultilanguage()
    {
        $de = \XLite\Core\Database::getRepo('XLite\Model\Language')->findOneBy(array('code' => 'de'));
        $de->setStatus(2);

        $en = \XLite\Core\Database::getRepo('XLite\Model\Language')->findOneBy(array('code' => 'en'));
        $en->setStatus(0);

        \XLite\Core\Database::getEM()->flush();

        \XLite\Core\Database::getCacheDriver()->deleteAll();

        $this->openAndWait('');

        $this->assertElementPresent(
            "//h1[@class='substitutional-test-skin' and text()='WELCOME PAGE DE']",
            'test DE substutional template'
        );

        $de->setStatus(0);
        $en->setStatus(2);
    }

    public function testInheritance()
    {
        copy(
            LC_DIR_SKINS . 'test' . LC_DS . 'en' . LC_DS . 'welcome.i.tpl',
            LC_DIR_SKINS . 'test' . LC_DS . 'en' . LC_DS . 'welcome.tpl'
        );
        $this->openAndWait('');

        $this->assertElementPresent(
            "//h1[@class='substitutional-test-skin' and text()='WELCOME PAGE']",
            'test substutional template'
        );

        $this->assertElementPresent(
            "//h3[text()='Access denied!']",
            'test inherited template'
        );
    }

    public function testDirectCall()
    {
        copy(
            LC_DIR_SKINS . 'test' . LC_DS . 'en' . LC_DS . 'welcome.d.tpl',
            LC_DIR_SKINS . 'test' . LC_DS . 'en' . LC_DS . 'welcome.tpl'
        );
        $this->openAndWait('');

        $this->assertElementPresent(
            "//h1[@class='substitutional-test-skin' and text()='WELCOME PAGE']",
            'test substutional template'
        );

        $this->assertElementPresent(
            "//h3[text()='Access denied!']",
            'test direct template'
        );
    }

}
